from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery


class QuantityQuery(BaseObjectQuery):
    """
    Class to query against a PIF Quantity object.
    """

    def __init__(self, logic=None, weight=None, simple=None, simple_weight=None, extract_as=None, extract_all=None,
                 extract_when_missing=None, tags=None, length=None, offset=None, actual_mass_percent=None,
                 actual_volume_percent=None, actual_number_percent=None, ideal_mass_percent=None,
                 ideal_volume_percent=None, ideal_number_percent=None, query=None, **kwargs):
        """
        Constructor.

        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param weight: Weight of the query.
        :param simple: String with the simple query to run against all fields.
        :param simple_weight: Dictionary of relative paths to their weights for simple queries.
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param tags: One or more :class:`FieldQuery` operations against the tags field.
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        :param actual_mass_percent: One or more :class:`FieldQuery` operations against the actual mass
        percent field.
        :param actual_volume_percent: One or more :class:`FieldQuery` operations against the actual volume
        percent field.
        :param actual_number_percent: One or more :class:`FieldQuery` operations against the actual number
        percent field.
        :param ideal_mass_percent: One or more :class:`FieldQuery` operations against the ideal mass
        percent field.
        :param ideal_volume_percent: One or more :class:`FieldQuery` operations against the ideal volume
        percent field.
        :param ideal_number_percent: One or more :class:`FieldQuery` operations against the ideal number
        percent field.
        :param query: One or more :class:`QuantityQuery` objects with nested queries.
        """
        super(QuantityQuery, self).__init__(
            logic=logic, weight=weight, simple=simple, simple_weight=simple_weight, extract_as=extract_as,
            extract_all=extract_all, extract_when_missing=extract_when_missing, tags=tags, length=length,
            offset=offset, **kwargs)
        self._actual_mass_percent = None
        self.actual_mass_percent = actual_mass_percent
        self._actual_volume_percent = None
        self.actual_volume_percent = actual_volume_percent
        self._actual_number_percent = None
        self.actual_number_percent = actual_number_percent
        self._ideal_mass_percent = None
        self.ideal_mass_percent = ideal_mass_percent
        self._ideal_volume_percent = None
        self.ideal_volume_percent = ideal_volume_percent
        self._ideal_number_percent = None
        self.ideal_number_percent = ideal_number_percent
        self._query = None
        self.query = query

    @property
    def actual_mass_percent(self):
        return self._actual_mass_percent

    @actual_mass_percent.setter
    def actual_mass_percent(self, actual_mass_percent):
        self._actual_mass_percent = self._get_object(FieldQuery, actual_mass_percent)

    @actual_mass_percent.deleter
    def actual_mass_percent(self):
        self._actual_mass_percent = None

    @property
    def actual_volume_percent(self):
        return self._actual_volume_percent

    @actual_volume_percent.setter
    def actual_volume_percent(self, actual_volume_percent):
        self._actual_volume_percent = self._get_object(FieldQuery, actual_volume_percent)

    @actual_volume_percent.deleter
    def actual_volume_percent(self):
        self._actual_volume_percent = None

    @property
    def actual_number_percent(self):
        return self._actual_number_percent

    @actual_number_percent.setter
    def actual_number_percent(self, actual_number_percent):
        self._actual_number_percent = self._get_object(FieldQuery, actual_number_percent)

    @actual_number_percent.deleter
    def actual_number_percent(self):
        self._actual_number_percent = None

    @property
    def ideal_mass_percent(self):
        return self._ideal_mass_percent

    @ideal_mass_percent.setter
    def ideal_mass_percent(self, ideal_mass_percent):
        self._ideal_mass_percent = self._get_object(FieldQuery, ideal_mass_percent)

    @ideal_mass_percent.deleter
    def ideal_mass_percent(self):
        self._ideal_mass_percent = None

    @property
    def ideal_volume_percent(self):
        return self._ideal_volume_percent

    @ideal_volume_percent.setter
    def ideal_volume_percent(self, ideal_volume_percent):
        self._ideal_volume_percent = self._get_object(FieldQuery, ideal_volume_percent)

    @ideal_volume_percent.deleter
    def ideal_volume_percent(self):
        self._ideal_volume_percent = None

    @property
    def ideal_number_percent(self):
        return self._ideal_number_percent

    @ideal_number_percent.setter
    def ideal_number_percent(self, ideal_number_percent):
        self._ideal_number_percent = self._get_object(FieldQuery, ideal_number_percent)

    @ideal_number_percent.deleter
    def ideal_number_percent(self):
        self._ideal_number_percent = None

    @property
    def query(self):
        return self._query

    @query.setter
    def query(self, query):
        self._query = self._get_object(QuantityQuery, query)

    @query.deleter
    def query(self):
        self._query = None



from pypif.util.serializable import Serializable


class FileSearchHit(Serializable):
    """
    Class to store a single file search hit.
    """

    def __init__(self, dataset_id=None, dataset_version=None, id=None, score=None, name=None, updated_at=None, 
                 highlights=None, **kwargs):
        """
        Constructor.

        :param dataset_id: String with the ID of the dataset.
        :param dataset_version: Long with the version of the dataset.
        :param id: String with the ID of the record.
        :param score: Score with the relevancy of the result.
        :param name: Name of the dataset.
        :param updated_at: String with the last time that the dataset was updated.
        :param highlights: List of strings with the highlighted results.
        """
        self._dataset_id = None
        self.dataset_id = dataset_id
        self._dataset_version = None
        self.dataset_version = dataset_version
        self._id = None
        self.id = id
        self._score = None
        self.score = score
        self._name = None
        self.name = name
        self._updated_at = None
        self.updated_at = updated_at
        self._highlights = None
        self.highlights = highlights

    @property
    def dataset_id(self):
        return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id):
        self._dataset_id = dataset_id

    @dataset_id.deleter
    def dataset_id(self):
        self._dataset_id = None

    @property
    def dataset_version(self):
        return self._dataset_version

    @dataset_version.setter
    def dataset_version(self, dataset_version):
        self._dataset_version = dataset_version

    @dataset_version.deleter
    def dataset_version(self):
        self._dataset_version = None

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @id.deleter
    def id(self):
        self._id = None

    @property
    def score(self):
        return self._score

    @score.setter
    def score(self, score):
        self._score = score

    @score.deleter
    def score(self):
        self._score = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @name.deleter
    def name(self):
        self._name = None

    @property
    def updated_at(self):
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        self._updated_at = updated_at

    @updated_at.deleter
    def updated_at(self):
        self._updated_at = None

    @property
    def highlights(self):
        return self._highlights

    @highlights.setter
    def highlights(self, highlights):
        self._highlights = highlights

    @highlights.deleter
    def highlights(self):
        self._highlights = None



from pypif.util.serializable import Serializable

from citrination_client.search.dataset.query.dataset_query import DatasetQuery
from citrination_client.search.file.query.file_query import FileQuery
from citrination_client.search.pif.query.pif_system_query import PifSystemQuery


class DataQuery(Serializable):
    """
    Query against dataset metadata, PIF content, file content, or some combination of those types.
    """

    def __init__(self, logic=None, weight=None, simple=None, simple_weight=None, dataset=None, system=None, file=None,
                 query=None, **kwargs):
        """
        Constructor.

        :param logic: The logic to apply to the query ('SHOULD', 'MUST', 'MUST_NOT', or 'OPTIONAL').
        :param weight: Weight for the query.
        :param simple: String with the simple search to run against all fields.
        :param simple_weight: Dictionary of relative paths to their weights for simple queries.
        :param dataset: One or more :class:`DatasetQuery` objects with queries against dataset metadata.
        :param system: One or more :class:`PifSystemQuery` objects with queries against PIF systems.
        :param file: One or more :class:`FileQuery` objects with queries against file content or metadata.
        :param query: Nested list of :class:`DataQuery` objects.
        """
        self._logic = None
        self.logic = logic
        self._weight = None
        self.weight = weight
        self._simple = None
        self.simple = simple
        self._simple_weight = None
        self.simple_weight = simple_weight
        self._dataset = None
        self.dataset = dataset
        self._system = None
        self.system = system
        self._file = None
        self.file = file
        self._query = None
        self.query = query

    @property
    def logic(self):
        return self._logic

    @logic.setter
    def logic(self, logic):
        self._logic = logic

    @logic.deleter
    def logic(self):
        self._logic = None

    @property
    def weight(self):
        return self._weight

    @weight.setter
    def weight(self, weight):
        self._weight = weight

    @weight.deleter
    def weight(self):
        self._weight = None

    @property
    def simple(self):
        return self._simple

    @simple.setter
    def simple(self, simple):
        self._simple = simple

    @simple.deleter
    def simple(self):
        self._simple = None

    @property
    def simple_weight(self):
        return self._simple_weight

    @simple_weight.setter
    def simple_weight(self, simple_weight):
        self._simple_weight = simple_weight

    @simple_weight.deleter
    def simple_weight(self):
        self._simple_weight = None

    @property
    def dataset(self):
        return self._dataset

    @dataset.setter
    def dataset(self, dataset):
        self._dataset = self._get_object(DatasetQuery, dataset)

    @dataset.deleter
    def dataset(self):
        self._dataset = None

    @property
    def system(self):
        return self._system

    @system.setter
    def system(self, system):
        self._system = self._get_object(PifSystemQuery, system)

    @system.deleter
    def system(self):
        self._system = None

    @property
    def file(self):
        return self._file

    @file.setter
    def file(self, file):
        self._file = self._get_object(FileQuery, file)

    @file.deleter
    def file(self):
        self._file = None

    @property
    def query(self):
        return self._query

    @query.setter
    def query(self, query):
        self._query = self._get_object(DataQuery, query)

    @query.deleter
    def query(self):
        self._query = None



from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery


class SourceQuery(BaseObjectQuery):
    """
    Class to query against a PIF Source object.
    """

    def __init__(self, logic=None, weight=None, simple=None, simple_weight=None, extract_as=None, extract_all=None,
                 extract_when_missing=None, tags=None, length=None, offset=None, producer=None, url=None,
                 query=None, **kwargs):
        """
        Constructor.

        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param weight: Weight of the query.
        :param simple: String with the simple query to run against all fields.
        :param simple_weight: Dictionary of relative paths to their weights for simple queries.
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param tags: One or more :class:`FieldQuery` operations against the tags field.
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        :param producer: One or more :class:`FieldQuery` operations against the producer field.
        :param url: One or more :class:`FieldQuery` operations against the url field.
        :param query: One or more :class:`SourceQuery` objects with nested queries.
        """
        super(SourceQuery, self).__init__(
            logic=logic, weight=weight, simple=simple, simple_weight=simple_weight, extract_as=extract_as,
            extract_all=extract_all, extract_when_missing=extract_when_missing, tags=tags, length=length,
            offset=offset, **kwargs)
        self._producer = None
        self.producer = producer
        self._url = None
        self.url = url
        self._query = None
        self.query = query

    @property
    def producer(self):
        return self._producer

    @producer.setter
    def producer(self, producer):
        self._producer = self._get_object(FieldQuery, producer)

    @producer.deleter
    def producer(self):
        self._producer = None

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, url):
        self._url = self._get_object(FieldQuery, url)

    @url.deleter
    def url(self):
        self._url = None

    @property
    def query(self):
        return self._query

    @query.setter
    def query(self, query):
        self._query = self._get_object(SourceQuery, query)

    @query.deleter
    def query(self):
        self._query = None



from pypif.util.serializable import Serializable


class ExtractionSort(Serializable):
    """
    Class to store information about a sort on an extracted field.
    """

    def __init__(self, key=None, order=None, **kwargs):
        """
        Constructor.

        :param key: String with the key that will be sorted on.
        :param order: The order to use. Either ASCENDING or DESCENDING.
        """
        self._key = None
        self.key = key
        self._order = None
        self.order = order

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, key):
        self._key = key

    @key.deleter
    def key(self):
        self._key = None

    @property
    def order(self):
        return self._order

    @order.setter
    def order(self, order):
        self._order = order

    @order.deleter
    def order(self):
        self._order = None



from citrination_client.search.pif.query.chemical.chemical_filter import ChemicalFilter
from citrination_client.search.pif.query.core.base_field_query import BaseFieldQuery


class ChemicalFieldQuery(BaseFieldQuery):
    """
    Class for all field operations against chemical information.
    """

    def __init__(self, sort=None, logic=None, weight=None, simple=None, simple_weight=None, extract_as=None,
                 extract_all=None, extract_when_missing=None, length=None, offset=None, filter=None, **kwargs):
        """
        Constructor.

        :param sort: ASCENDING or DESCENDING to set the sort order on this field.
        :param logic: Logic for this query. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param weight: Weight for the query.
        :param simple: String with the simple search to run against all fields.
        :param simple_weight: Dictionary of relative paths to their weights for simple queries.
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param length: One or more :class:`FieldOperation` objects against the length field.
        :param offset: One or more :class:`FieldOperation` objects against the offset field.
        :param filter: One or more :class:`ChemicalFilter` objects against this field.
        """
        super(ChemicalFieldQuery, self).__init__(
            sort=sort, weight=weight, logic=logic, simple=simple, simple_weight=simple_weight, extract_as=extract_as,
            extract_all=extract_all, extract_when_missing=extract_when_missing, length=length, offset=offset, **kwargs)
        self._filter = None
        self.filter = filter

    @property
    def filter(self):
        return self._filter

    @filter.setter
    def filter(self, filter):
        self._filter = self._get_object(ChemicalFilter, filter)

    @filter.deleter
    def filter(self):
        self._filter = None



from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery
from citrination_client.search.pif.query.core.file_reference_query import FileReferenceQuery


class ValueQuery(BaseObjectQuery):
    """
    Class to query against a single value.
    """

    def __init__(self, logic=None, weight=None, simple=None, simple_weight=None, extract_as=None, extract_all=None,
                 extract_when_missing=None, tags=None, length=None, offset=None, name=None, value=None,
                 file=None, units=None, query=None, **kwargs):
        """
        Constructor.

        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param weight: Weight of the query.
        :param simple: String with the simple query to run against all fields.
        :param simple_weight: Dictionary of relative paths to their weights for simple queries.
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param tags: One or more :class:`FieldQuery` operations against the tags field.
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        :param name: One or more :class:`FieldQuery` operations against the name field.
        :param value: One or more :class:`FieldQuery` operations against the value.
        :param file: One or more :class:`FileReferenceQuery` operations against the file.
        :param units: One or more :class:`FieldQuery` operations against the units field.
        :param query: One or more :class:`ValueQuery` objects with nested queries.
        """
        super(ValueQuery, self).__init__(
            logic=logic, weight=weight, simple=simple, simple_weight=simple_weight, extract_as=extract_as,
            extract_all=extract_all, extract_when_missing=extract_when_missing, tags=tags, length=length,
            offset=offset, **kwargs)
        self._name = None
        self.name = name
        self._value = None
        self.value = value
        self._file = None
        self.file = file
        self._units = None
        self.units = units
        self._query = None
        self.query = query

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = self._get_object(FieldQuery, name)

    @name.deleter
    def name(self):
        self._name = None

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = self._get_object(FieldQuery, value)

    @value.deleter
    def value(self):
        self._value = None

    @property
    def file(self):
        return self._file

    @file.setter
    def file(self, file):
        self._file = self._get_object(FileReferenceQuery, file)

    @file.deleter
    def file(self):
        self._file = None

    @property
    def units(self):
        return self._units

    @units.setter
    def units(self, units):
        self._units = self._get_object(FieldQuery, units)

    @units.deleter
    def units(self):
        self._units = None

    @property
    def query(self):
        return self._query

    @query.setter
    def query(self, query):
        self._query = self._get_object(ValueQuery, query)

    @query.deleter
    def query(self):
        self._query = None



from citrination_client.search.core.query.base_returning_query import BaseReturningQuery
from citrination_client.search.pif.query.extraction_sort import ExtractionSort


class PifSystemReturningQuery(BaseReturningQuery):
    """
    Query used to return information about PIFs.
    """

    def __init__(self, query=None, extraction_sort=None, from_index=None, size=None, random_results=None,
                 random_seed=None, score_relevance=None, return_max_score=None, timeout=None, return_system=None,
                 add_latex=None, return_extracted_path=None, unwrap_single_value_extractions=None, **kwargs):
        """
        Constructor.

        :param query: One or more :class:`DataQuery` objects with the queries to run.
        :param extraction_sort: A single :class:`ExtractionSort` object for sorting.
        :param from_index: Index of the first hit that should be returned.
        :param size: Total number of hits the should be returned.
        :param random_results: Whether to return a random set of records.
        :param random_seed: The random seed to use.
        :param score_relevance: Whether to use relevance scoring.
        :param return_max_score: Whether to return the maximum score.
        :param timeout: The number of milliseconds to wait for the query to execute.
        :param return_system: Whether to return the matched PIF systems.
        :param add_latex: Whether to add latex formatting where possible in results.
        :param return_extracted_path: Whether to return the path in PIFs for extracted values.
        :param unwrap_single_value_extractions: Whether to unwrap extracted values when they are lists with one value.
        """
        super(PifSystemReturningQuery, self).__init__(
            query=query, from_index=from_index, size=size, random_results=random_results, random_seed=random_seed,
            score_relevance=score_relevance, return_max_score=return_max_score, timeout=timeout, **kwargs)
        self._return_system = None
        self.return_system = return_system
        self._add_latex = None
        self.add_latex = add_latex
        self._return_extracted_path = None
        self.return_extracted_path = return_extracted_path
        self._unwrap_single_value_extractions = None
        self.unwrap_single_value_extractions = unwrap_single_value_extractions
        self._extraction_sort = None
        self.extraction_sort = extraction_sort

    @property
    def return_system(self):
        return self._return_system

    @return_system.setter
    def return_system(self, return_system):
        self._return_system = return_system

    @return_system.deleter
    def return_system(self):
        self._return_system = None

    @property
    def add_latex(self):
        return self._add_latex

    @add_latex.setter
    def add_latex(self, add_latex):
        self._add_latex = add_latex

    @add_latex.deleter
    def add_latex(self):
        self._add_latex = None

    @property
    def return_extracted_path(self):
        return self._return_extracted_path

    @return_extracted_path.setter
    def return_extracted_path(self, return_extracted_path):
        self._return_extracted_path = return_extracted_path

    @return_extracted_path.deleter
    def return_extracted_path(self):
        self._return_extracted_path = None

    @property
    def unwrap_single_value_extractions(self):
        return self._unwrap_single_value_extractions

    @unwrap_single_value_extractions.setter
    def unwrap_single_value_extractions(self, unwrap_single_value_extractions):
        self._unwrap_single_value_extractions = unwrap_single_value_extractions

    @unwrap_single_value_extractions.deleter
    def unwrap_single_value_extractions(self):
        self._unwrap_single_value_extractions = None

    @property
    def extraction_sort(self):
        return self._extraction_sort

    @extraction_sort.setter
    def extraction_sort(self, extraction_sort):
        self._extraction_sort = self._get_object(ExtractionSort, extraction_sort)

    @extraction_sort.deleter
    def extraction_sort(self):
        self._extraction_sort = None



class CitrinationClientError(Exception):
  pass

class RequestTimeoutException(CitrinationClientError):

  def __init__(self, message="Request to Citrination timed out"):
    super(RequestTimeoutException, self).__init__(message)


from citrination_client.design.constraints.base import BaseConstraint
from citrination_client.errors import CitrinationClientError

class ElementalInclusionConstraint(BaseConstraint):
    """
    Constraint class which allows the assertion that a set of 
    elements is included in the candidate compositions
    """

    def __init__(self, name, elements, logic):
        """
        Constructor.

        :param name: The name of the column in the data
            view to which this constraint should be applied
        :type name: str
        :param elements: An array of element abbreviations as
            strings, e.g. ["Mg", "C"]
        :type elements: list of str
        :param logic: The logic to apply to the constraint; either
            "must", "should", or "exclude"
        :type logic: str
        """
        bad_logic_msg = "ElementalInclusionConstraint must be initialized with the logic parameter equal to \"must\", \"should\", or \"exclude\""

        if logic not in ["must", "should", "exclude"]:
            raise CitrinationClientError(bad_logic_msg)

        self._name = name
        self._type = "elementalInclusionConstraint"
        self._elements = elements
        self._logic = logic

    def options(self):
        return {
            "elements": self._elements,
            "logic": self._logic
        }


from pypif.util.serializable import Serializable


class DatasetSearchHit(Serializable):
    """
    Class to store a single dataset search hit.
    """

    def __init__(self, id=None, score=None, is_featured=None, name=None, description=None, owner=None, email=None, 
                 num_pifs=None, updated_at=None, **kwargs):
        """
        Constructor.

        :param id: String with the ID of the record.
        :param score: Score with the relevancy of the result.
        :param is_featured: Whether the dataset is a featured one.
        :param name: Name of the dataset.
        :param description: Description of the dataset.
        :param owner: Name of the owner of the dataset.
        :param email: Email address of the owner of the dataset.
        :param num_pifs: Number of PIFs in the dataset.
        :param updated_at: String with the last time that the dataset was updated.
        """
        self._id = None
        self.id = id
        self._score = None
        self.score = score
        self._is_featured = None
        self.is_featured = is_featured
        self._name = None
        self.name = name
        self._description = None
        self.description = description
        self._owner = None
        self.owner = owner
        self._email = None
        self.email = email
        self._num_pifs = None
        self.num_pifs = num_pifs
        self._updated_at = None
        self.updated_at = updated_at

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @id.deleter
    def id(self):
        self._id = None

    @property
    def score(self):
        return self._score

    @score.setter
    def score(self, score):
        self._score = score

    @score.deleter
    def score(self):
        self._score = None

    @property
    def is_featured(self):
        return self._is_featured

    @is_featured.setter
    def is_featured(self, is_featured):
        self._is_featured = is_featured

    @is_featured.deleter
    def is_featured(self):
        self._is_featured = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @name.deleter
    def name(self):
        self._name = None

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, description):
        self._description = description

    @description.deleter
    def description(self):
        self._description = None

    @property
    def owner(self):
        return self._owner

    @owner.setter
    def owner(self, owner):
        self._owner = owner

    @owner.deleter
    def owner(self):
        self._owner = None

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, email):
        self._email = email

    @email.deleter
    def email(self):
        self._email = None

    @property
    def num_pifs(self):
        return self._num_pifs

    @num_pifs.setter
    def num_pifs(self, num_pifs):
        self._num_pifs = num_pifs

    @num_pifs.deleter
    def num_pifs(self):
        self._num_pifs = None

    @property
    def updated_at(self):
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        self._updated_at = updated_at

    @updated_at.deleter
    def updated_at(self):
        self._updated_at = None



from citrination_client.search.pif.query.chemical.chemical_field_query import ChemicalFieldQuery
from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery


class CompositionQuery(BaseObjectQuery):
    """
    Class to query against a PIF Composition object.
    """

    def __init__(self, logic=None, weight=None, simple=None, simple_weight=None, extract_as=None,
                 extract_all=None, extract_when_missing=None, tags=None, length=None, offset=None, element=None,
                 actual_weight_percent=None, actual_atomic_percent=None, ideal_weight_percent=None,
                 ideal_atomic_percent=None, query=None, **kwargs):
        """
        Constructor.

        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param weight: Weight of the query.
        :param simple: String with the simple query to run against all fields.
        :param simple_weight: Dictionary of relative paths to their weights for simple queries.
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param tags: One or more :class:`FieldQuery` operations against the tags field.
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        :param element: One or more :class:`ChemicalFieldQuery` operations against the element field.
        :param actual_weight_percent: One or more :class:`FieldQuery` operations against the actual
        weight percent field.
        :param actual_atomic_percent: One or more :class:`FieldQuery` operations against the actual
        atomic percent field.
        :param ideal_weight_percent: One or more :class:`FieldQuery` operations against the ideal
        weight percent field.
        :param ideal_atomic_percent: One or more :class:`FieldQuery` operations against the ideal
        atomic percent field.
        :param query: One or more :class:`CompositionQuery` objects with the nest queries.
        """
        super(CompositionQuery, self).__init__(
            logic=logic, weight=weight, simple=simple, simple_weight=simple_weight, extract_as=extract_as,
            extract_all=extract_all, extract_when_missing=extract_when_missing, tags=tags, length=length,
            offset=offset, **kwargs)
        self._element = None
        self.element = element
        self._actual_weight_percent = None
        self.actual_weight_percent = actual_weight_percent
        self._actual_atomic_percent = None
        self.actual_atomic_percent = actual_atomic_percent
        self._ideal_weight_percent = None
        self.ideal_weight_percent = ideal_weight_percent
        self._ideal_atomic_percent = None
        self.ideal_atomic_percent = ideal_atomic_percent
        self._query = None
        self.query = query

    @property
    def element(self):
        return self._element

    @element.setter
    def element(self, element):
        self._element = self._get_object(ChemicalFieldQuery, element)

    @element.deleter
    def element(self):
        self._element = None

    @property
    def actual_weight_percent(self):
        return self._actual_weight_percent

    @actual_weight_percent.setter
    def actual_weight_percent(self, actual_weight_percent):
        self._actual_weight_percent = self._get_object(FieldQuery, actual_weight_percent)

    @actual_weight_percent.deleter
    def actual_weight_percent(self):
        self._actual_weight_percent = None

    @property
    def actual_atomic_percent(self):
        return self._actual_atomic_percent

    @actual_atomic_percent.setter
    def actual_atomic_percent(self, actual_atomic_percent):
        self._actual_atomic_percent = self._get_object(FieldQuery, actual_atomic_percent)

    @actual_atomic_percent.deleter
    def actual_atomic_percent(self):
        self._actual_atomic_percent = None

    @property
    def ideal_weight_percent(self):
        return self._ideal_weight_percent

    @ideal_weight_percent.setter
    def ideal_weight_percent(self, ideal_weight_percent):
        self._ideal_weight_percent = self._get_object(FieldQuery, ideal_weight_percent)

    @ideal_weight_percent.deleter
    def ideal_weight_percent(self):
        self._ideal_weight_percent = None

    @property
    def ideal_atomic_percent(self):
        return self._ideal_atomic_percent

    @ideal_atomic_percent.setter
    def ideal_atomic_percent(self, ideal_atomic_percent):
        self._ideal_atomic_percent = self._get_object(FieldQuery, ideal_atomic_percent)

    @ideal_atomic_percent.deleter
    def ideal_atomic_percent(self):
        self._ideal_atomic_percent = None

    @property
    def query(self):
        return self._query

    @query.setter
    def query(self, query):
        self._query = self._get_object(CompositionQuery, query)

    @query.deleter
    def query(self):
        self._query = None



from pypif.util.serializable import Serializable


class MultiQuery(Serializable):
    """
    Base class for all multi-search requests.
    """

    def __init__(self, queries=None, **kwargs):
        """
        Constructor.

        :param queries: One or more queries to run.
        """
        self._queries = None
        self.queries = queries

    @property
    def queries(self):
        return self._queries

    @queries.setter
    def queries(self, queries):
        self._queries = queries

    @queries.deleter
    def queries(self):
        self._queries = None



from citrination_client.errors import CitrinationClientError

class Target(object):
    """
    The optimization target for a design run. Consists of
    the name of the output column to optimize and the objective
    (either "Max" or "Min")
    """

    def __init__(self, name, objective):
        """
        Constructor.

        :param name: The name of the target output column
        :type name: str
        :param objective: The optimization objective; either "Min"
            or "Max"
        :type objective: str
        """
        if objective not in ["Max", "Min"]:
            raise CitrinationClientError(
                    "Target objective must either be \"Min\" or \"Max\""
                )

        self._name = name
        self._objective = objective

    def to_dict(self):
        return {
            "descriptor": self._name,
            "objective": self._objective
        }


from citrination_client.design.constraints.base import BaseConstraint
from citrination_client.design.constraints.real_value import RealValueConstraint
from citrination_client.design.constraints.real_range import RealRangeConstraint
from citrination_client.design.constraints.categorical import CategoricalConstraint
from citrination_client.design.constraints.elemental_inclusion import ElementalInclusionConstraint
from citrination_client.design.constraints.elemental_composition import ElementalCompositionConstraint



from citrination_client.design.design_run import DesignRun
from citrination_client.design.process_status import ProcessStatus
from citrination_client.design.design_results import DesignResults
from citrination_client.design.target import Target
from citrination_client.design.constraints import *


from citrination_client.search.core.query.boolean_filter import BooleanFilter
from citrination_client.search.core.query.data_query import DataQuery
from citrination_client.search.core.query.data_scope import DataScope
from citrination_client.search.core.query.filter import Filter
from citrination_client.search.core.query.multi_query import MultiQuery
from citrination_client.search.dataset.query.dataset_query import DatasetQuery
from citrination_client.search.dataset.query.dataset_returning_query import DatasetReturningQuery
from citrination_client.search.dataset.result.dataset_multi_search_result import DatasetMultiSearchResult
from citrination_client.search.dataset.result.dataset_multi_search_result_element import DatasetMultiSearchResultElement
from citrination_client.search.dataset.result.dataset_search_hit import DatasetSearchHit
from citrination_client.search.dataset.result.dataset_search_result import DatasetSearchResult
from citrination_client.search.file.query.file_query import FileQuery
from citrination_client.search.file.query.file_returning_query import FileReturningQuery
from citrination_client.search.file.result.file_multi_search_result import FileMultiSearchResult
from citrination_client.search.file.result.file_multi_search_result_element import FileMultiSearchResultElement
from citrination_client.search.file.result.file_search_hit import FileSearchHit
from citrination_client.search.file.result.file_search_result import FileSearchResult
from citrination_client.search.pif.query.chemical.chemical_field_query import ChemicalFieldQuery
from citrination_client.search.pif.query.chemical.chemical_filter import ChemicalFilter
from citrination_client.search.pif.query.chemical.composition_query import CompositionQuery
from citrination_client.search.pif.query.core.base_field_query import BaseFieldQuery
from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.classification_query import ClassificationQuery
from citrination_client.search.pif.query.core.display_item_query import DisplayItemQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery
from citrination_client.search.pif.query.core.file_reference_query import FileReferenceQuery
from citrination_client.search.pif.query.core.id_query import IdQuery
from citrination_client.search.pif.query.core.name_query import NameQuery
from citrination_client.search.pif.query.core.pages_query import PagesQuery
from citrination_client.search.pif.query.core.process_step_query import ProcessStepQuery
from citrination_client.search.pif.query.core.property_query import PropertyQuery
from citrination_client.search.pif.query.core.quantity_query import QuantityQuery
from citrination_client.search.pif.query.core.reference_query import ReferenceQuery
from citrination_client.search.pif.query.core.source_query import SourceQuery
from citrination_client.search.pif.query.core.value_query import ValueQuery
from citrination_client.search.pif.query.extraction_sort import ExtractionSort
from citrination_client.search.pif.query.pif_system_query import PifSystemQuery
from citrination_client.search.pif.query.pif_system_returning_query import PifSystemReturningQuery
from citrination_client.search.pif.result.pif_multi_search_result import PifMultiSearchResult
from citrination_client.search.pif.result.pif_multi_search_result_element import PifMultiSearchResultElement
from citrination_client.search.pif.result.pif_search_hit import PifSearchHit
from citrination_client.search.pif.result.pif_search_result import PifSearchResult



from setuptools import setup, find_packages

setup(name='citrination-client',
      version='3.4.0',
      url='http://github.com/CitrineInformatics/python-citrination-client',
      description='Python client for accessing the Citrination api',
      packages=find_packages(exclude=('docs')),
      install_requires=[
          'requests<3',
          'pypif',
          'six<2'
      ])



from pypif.util.serializable import Serializable


class BaseFieldQuery(Serializable):
    """
    Base class for all field queries.
    """

    def __init__(self, sort=None, logic=None, weight=None, simple=None, simple_weight=None, extract_as=None,
                 extract_all=None, extract_when_missing=None, length=None, offset=None, **kwargs):
        """
        Constructor.

        :param sort: ASCENDING or DESCENDING to set the sort order on this field.
        :param logic: Logic for this query. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param weight: Weight for the query.
        :param simple: String with the simple search to run against all fields.
        :param simple_weight: Dictionary of relative paths to their weights for simple queries.
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        """
        self._sort = None
        self.sort = sort
        self._logic = None
        self.logic = logic
        self._weight = None
        self.weight = weight
        self._simple = None
        self.simple = simple
        self._simple_weight = None
        self.simple_weight = simple_weight
        self._extract_as = None
        self.extract_as = extract_as
        self._extract_all = None
        self.extract_all = extract_all
        self._extract_when_missing = None
        self.extract_when_missing = extract_when_missing
        self._length = None
        self.length = length
        self._offset = None
        self.offset = offset

    @property
    def sort(self):
        return self._sort

    @sort.setter
    def sort(self, sort):
        self._sort = sort

    @sort.deleter
    def sort(self):
        self._sort = None

    @property
    def logic(self):
        return self._logic

    @logic.setter
    def logic(self, logic):
        self._logic = logic

    @logic.deleter
    def logic(self):
        self._logic = None

    @property
    def weight(self):
        return self._weight

    @weight.setter
    def weight(self, weight):
        self._weight = weight

    @weight.deleter
    def weight(self):
        self._weight = None

    @property
    def simple(self):
        return self._simple

    @simple.setter
    def simple(self, simple):
        self._simple = simple

    @simple.deleter
    def simple(self):
        self._simple = None

    @property
    def simple_weight(self):
        return self._simple_weight

    @simple_weight.setter
    def simple_weight(self, simple_weight):
        self._simple_weight = simple_weight

    @simple_weight.deleter
    def simple_weight(self):
        self._simple_weight = None

    @property
    def extract_as(self):
        return self._extract_as

    @extract_as.setter
    def extract_as(self, extract_as):
        self._extract_as = extract_as

    @extract_as.deleter
    def extract_as(self):
        self._extract_as = None

    @property
    def extract_all(self):
        return self._extract_all

    @extract_all.setter
    def extract_all(self, extract_all):
        self._extract_all = extract_all

    @extract_all.deleter
    def extract_all(self):
        self._extract_all = None

    @property
    def extract_when_missing(self):
        return self._extract_when_missing

    @extract_when_missing.setter
    def extract_when_missing(self, extract_when_missing):
        self._extract_when_missing = extract_when_missing

    @extract_when_missing.deleter
    def extract_when_missing(self):
        self._extract_when_missing = None

    @property
    def length(self):
        return self._length

    @length.setter
    def length(self, length):
        from citrination_client.search.pif.query.core.field_query import FieldQuery
        self._length = self._get_object(FieldQuery, length)

    @length.deleter
    def length(self):
        self._length = None

    @property
    def offset(self):
        return self._offset

    @offset.setter
    def offset(self, offset):
        from citrination_client.search.pif.query.core.field_query import FieldQuery
        self._offset = self._get_object(FieldQuery, offset)

    @offset.deleter
    def offset(self):
        self._offset = None



from citrination_client.design.constraints.base import BaseConstraint
from citrination_client.errors import CitrinationClientError

class RealRangeConstraint(BaseConstraint):
    """
    Constrains a real valued column to a range.
    """

    def __init__(self, name, minimum, maximum):
        """
        Constructor.

        :param name: The name of the column in the data
            view to which this constraint should be applied
        :type name: str
        :param minimum: The minimum allowed value
        :type minimum: float
        :param maximum: The maximum allowed value
        :type maximum: float
        """

        try:
            minimum_f = float(minimum)
            maximum_f = float(maximum)
        except TypeError:
            raise CitrinationClientError("RealRangeConstraint requires that minimum and maximum must be able to be cast to float")

        if minimum_f > maximum_f:
            raise CitrinationClientError("RealRangeConstraint requires that minimum be less than maximum")

        self._min = minimum_f
        self._max = maximum_f
        self._type = "real"
        self._name = name

    def options(self):
        return {
            "min": self._min,
            "max": self._max
        }